@@ -31,6 +31,7 @@ import shutil
 import ldap
 import socket
 import time
+import stat
 from datetime import datetime
 import sys
 import filecmp
@@ -308,7 +309,10 @@ def selinux_label_port(port, remove_label=False):
                                          "-p", "tcp", str(port)],
                                          stdout=subprocess.PIPE,
                                          stderr=subprocess.PIPE)
-                log.debug(f"CMD: {' '.join(result.args)} ; STDOUT: {result.stdout} ; STDERR: {result.stderr}")
+                args = ' '.join(ensure_list_str(result.args))
+                stdout = ensure_str(result.stdout)
+                stderr = ensure_str(result.stderr)
+                log.debug(f"CMD: {args} ; STDOUT: {stdout} ; STDERR: {stderr}")
                 return
             except (OSError, subprocess.CalledProcessError) as e:
                 label_ex = e
@@ -1249,9 +1253,10 @@ def get_ldapurl_from_serverid(instance):
         return ("ldap://{}:{}".format(host, port), None)
 
 
-def get_instance_list(prefix=None):
+def get_instance_list():
     # List all server instances
-    conf_dir = (prefix or "") + "/etc/dirsrv/"
+    paths = Paths()
+    conf_dir = os.path.join(paths.sysconf_dir, 'dirsrv')
     insts = []
     try:
         for inst in os.listdir(conf_dir):
@@ -1259,6 +1264,9 @@ def get_instance_list(prefix=None):
                 insts.append(inst)
     except OSError as e:
         log.error("Failed to check directory: {} - {}".format(conf_dir, str(e)))
+    except IOError as e:
+        log.error(e)
+        log.error("Perhaps you need to be a different user?")
     insts.sort()
     return insts
 
@@ -1360,3 +1368,12 @@ def print_nice_time(seconds):
         return f'{m:d} minute{m_plural}, {s:d} second{s_plural}'
     else:
         return f'{s:d} second{s_plural}'
+
+
+def copy_with_permissions(source, target):
+    """Copy a file while preserving all permissions"""
+
+    shutil.copy2(source, target)
+    st = os.stat(source)
+    os.chown(target, st[stat.ST_UID], st[stat.ST_GID])
+    os.chmod(target, st[stat.ST_MODE])
